local xx, yy, rr
local canvas

function love.load ( )
	xx, yy, _ = love.window.getMode ( )
	xx, yy = xx / 2, yy / 2
	rr = math.min ( xx, yy ) * 0.75
	canvas = love.graphics.newCanvas ( rr * 4, rr * 2 )
end

function love.resize ( )
	xx, yy, _ = love.window.getMode ( )
	xx, yy = xx / 2, yy / 2
	rr = math.min ( xx, yy ) * 0.75
	canvas = love.graphics.newCanvas ( rr * 4, rr * 2 )
end

local wheelfunc
local band_density = 4
local sector_density = 4
function love.keypressed ( key, code, repeated )
	if code == 'escape' then
		love.event.quit ( )
	end

	if code == 'up' then
		band_density = band_density + 1
	elseif code == 'down' and band_density > 1 then
		band_density = band_density - 1
	end
	if code == 'right' then
		sector_density = sector_density + 1
	elseif code == 'left' and sector_density > 1 then
		sector_density = sector_density - 1
	end
	if code == 'space' then
		if wheelfunc == 'classic' then
			wheelfunc = 'rainbow'
		else
			wheelfunc = 'classic'
		end
	end
	if code == 'r' then
		band_density = 4
		sector_density = 4
	end
end

local rot = 0
local vel = 0
local delta = 0
function love.update ( dt )
	delta = dt
	rot = ( rot + vel * dt ) % ( math.pi * 2 )
end

function love.mousemoved ( x, y, dx, dy, touch )
	vel = math.min ( 1, math.max ( -1, ( xx - x ) / rr ) ) * 200
end

local function draw_wheel_rainbow ( )
	-- settings
	local outer_band = 0.95
	local inner_band = 0.10
	local sector_length = ( math.pi * 1.25 ) / sector_density --math.pi / 3
	local num_bands = band_density * 6 --32

	local band_width = ( outer_band - inner_band ) / ( num_bands * 2 )
	local sector_step = ( math.pi - sector_length ) / ( num_bands - 1 )

	local rr = rr * 2
	love.graphics.setColor ( 1, 1, 1 )
	love.graphics.circle ( "fill", rr, rr, rr )

	for band = 0, num_bands - 1 do
		love.graphics.setColor ( 0, 0, 0 )
		love.graphics.arc ( "fill", rr, rr, rr * ( outer_band - band * 2 * band_width ), math.pi + band * sector_step, math.pi + band * sector_step + sector_length )
		love.graphics.setColor ( 1, 1, 1 )
		love.graphics.arc ( "fill", rr, rr, rr * ( outer_band - ( band * 2 + 1 ) * band_width ), math.pi + band * sector_step, math.pi + band * sector_step + sector_length )
	end
end

local function draw_wheel_sector ( )
	-- settings
	local sector_gap = 0.01
	local outer_band = 0.95
	local inner_band = 0.10
	local num_sectors = sector_density --4
	local num_bands = band_density --4

	local band_width = ( ( outer_band - inner_band ) / ( num_sectors + 1 ) ) / ( num_bands * 2 )
	local sector_step = ( outer_band - inner_band ) / num_sectors

	local rr = rr * 2
	love.graphics.setColor ( 1, 1, 1 )
	love.graphics.circle ( "fill", rr, rr, rr )

	for band = 0, num_bands - 1 do
		love.graphics.setColor ( 0, 0, 0 )
		for sector = 0, num_sectors - 1 do
			love.graphics.arc ( "fill", rr, rr, rr * ( outer_band - sector * sector_step - band_width * band * 2 ), math.pi * ( 1 + sector / num_sectors ) + sector_gap, math.pi * ( 1 + ( sector + 1 ) / num_sectors ) - sector_gap )
		end
		love.graphics.setColor ( 1, 1, 1 )
		for sector = 0, num_sectors - 1 do
			love.graphics.arc ( "fill", rr, rr, rr * ( outer_band - sector * sector_step - band_width * ( band * 2 + 1 ) ), math.pi * ( 1 + sector / num_sectors ) + sector_gap, math.pi * ( 1 + ( sector + 1 ) / num_sectors ) - sector_gap )
		end
	end
end

wheelfunc = 'classic'

function love.draw ( )
	love.graphics.clear ( 0.5, 0.5, 0.5 )
	love.graphics.setColor ( 0, 0, 0 )
	love.graphics.circle ( "fill", xx, yy, rr )

	love.graphics.print ( "use mouse to control speed\nleft-right: sector length\nup-down: band width\nspace: toggle wheel mode\nr: reset" )

	love.graphics.setCanvas ( canvas )
	love.graphics.clear ( 0, 0, 0, 0 )
	if wheelfunc == 'classic' then
		draw_wheel_sector ( )
	else
		draw_wheel_rainbow ( )
	end
	love.graphics.setCanvas ( )

	local blur = rr * vel * delta * 0.2
	local blurpasses = math.pow ( 2, math.ceil ( math.log ( math.abs ( blur ) ) / math.log ( 2 ) ) )
	blurpasses = math.min ( 128, math.max ( 1, blurpasses ) )

	love.graphics.setBlendMode ( 'add' )
	love.graphics.setColor ( 1, 1, 1, 1 / ( blurpasses - 1 ) )
	for i = 1, blurpasses do
		love.graphics.draw ( canvas, xx, yy, rot + vel * delta / blurpasses * i, 0.5, 0.5, rr * 2, rr * 2 )
	end
	love.graphics.setBlendMode ( 'alpha' )

end
